/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: Listener.java 464373 2006-10-16 04:21:54Z rahul $
 */

package org.apache.shale.examples.sqlbrowser;

import java.io.File;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.faces.FacesException;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.sql.DataSource;

/**
 * <p>A servlet context listener that sets up the embedded example database.</p>
 */
public class Listener implements ServletContextListener {


    // ------------------------------------------------------ Manifest Constants


    /**
     * <p>The application scope attribute under which we store a data source
     * for the embedded database.</p>
     */
    public static final String INTERNAL_DATA_SOURCE = "internal";



    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The internal data source we have configured for Derby.</p>
     */
    private InternalDataSource ds = null;


    /**
     * <p>The log instance for this class.</p>
     */
    private transient Logger logger =
      Logger.getLogger(this.getClass().getName());



    // ------------------------------------------ ServletContextListener Methods


    /**
     * <p>Perform cleanup actions upon application shutdown.</p>
     */
    public void contextDestroyed(ServletContextEvent event) {

        if (logger.isLoggable(Level.INFO)) {
            logger.log(Level.INFO, "Shutting down database");
        }

        // Shut down the Derby data source
        try {
            ds.shutdown();
        } catch (SQLException e) {
            if (logger.isLoggable(Level.SEVERE)) {
                while (e != null) {
                    logger.log(Level.SEVERE, e.getMessage(), e);
                    e = e.getNextException();
                }
            }
        }

    }


    /**
     * <p>Perform setup actions upon application startup.</p>
     */
    public void contextInitialized(ServletContextEvent event) {

        // Calculate a database URL for the database to be created
        ServletContext context = event.getServletContext();
        File tempDir = (File) context.getAttribute("javax.servlet.context.tempdir");
        String url = tempDir.getAbsolutePath() + File.separator + "DATABASE";
        if (logger.isLoggable(Level.INFO)) {
            logger.log(Level.INFO, "Creating database " + url);
        }

        // Create and publish a data source for the embedded database
        try {
            ds = new InternalDataSource(url);
            populate(ds);
        } catch (SQLException e) {
            if (logger.isLoggable(Level.SEVERE)) {
                while (e != null) {
                    logger.log(Level.SEVERE, e.getMessage(), e);
                    e = e.getNextException();
                }
            }
            throw new FacesException("SQLException occurred during startup (see log for details): " + e.getMessage());
        }
        context.setAttribute(INTERNAL_DATA_SOURCE, ds);

    }


    // ----------------------------------------------------- Private Static Data


    /**
     * <p>Initialization commands for the internal database.  If an exception
     * occurs on the first one (presumed to be a CREATE TABLE), then the
     * database is assumed to be already populated.</p>
     */
    private static final String populate[] = {
        "create table zip_codes (" +
          "zip_code           varchar(10)," +
          "city               varchar(30)," +
          "state              varchar(2)" +
          ")",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97005', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97006', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97007', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97008', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97075', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97076', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97077', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97078', 'Beaverton', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97034', 'Lake Oswego', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97035', 'Lake Oswego', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97062', 'Tualatin', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97068', 'West Linn', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97140', 'Sherwood', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97223', 'Tigard', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97224', 'Tigard', 'OR')",
        "insert into zip_codes (zip_code, city, state) " +
          "values ('97281', 'Tigard', 'OR')",
    };



    // --------------------------------------------------------- Private Methods


    /**
     * <p>Populate the internal database with dummy data.</p>
     *
     * @param ds <code>DataSource</code> for the internal database
     *
     * @exception SQLException if a database error occurs
     */
    private void populate(DataSource ds) throws SQLException {

        int i = -1;
        Connection conn = null;
        Statement stmt = null;
        try {
            conn = ds.getConnection();
            for (i = 0; i < populate.length; i++) {
                System.err.println("populate():  Executing: " + populate[i]);
                stmt = conn.createStatement();
                stmt.executeUpdate(populate[i]);
                stmt.close();
            }
        } catch (SQLException e) {
            if (i == 0) {
                return; // Database is already populated
            }
            throw e;
        } finally {
            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException e) {
                    ;
                }
            }
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    ;
                }
            }
        }

    }


}
